@@ -57,6 +57,7 @@ static CreateFileDialogHelperFunc createFileDialogHelper = nullptr;
 LXQtPlatformTheme::LXQtPlatformTheme():
     iconFollowColorScheme_(true)
     , settingsWatcher_(nullptr)
+    , LXQtPalette_(nullptr)
 {
     loadSettings();
     // Note: When the plugin is loaded, it seems that the app is not yet running and
@@ -75,8 +76,8 @@ LXQtPlatformTheme::LXQtPlatformTheme():
 }
 
 LXQtPlatformTheme::~LXQtPlatformTheme() {
-    if(settingsWatcher_)
-        delete settingsWatcher_;
+    delete LXQtPalette_;
+    delete settingsWatcher_;
 }
 
 void LXQtPlatformTheme::lazyInit()
@@ -113,6 +114,108 @@ void LXQtPlatformTheme::loadSettings() {
     // single click activation
     singleClickActivate_ = settings.value(QLatin1String("single_click_activate")).toBool();
 
+    // palette
+    settings.beginGroup(QLatin1String("Palette"));
+    paletteChanged_ = false;
+
+    QColor color = winColor_;
+    winColor_.setNamedColor(settings.value(QLatin1String("window_color"), QLatin1String("#efefef")).toString());
+    if(!winColor_.isValid())
+        winColor_.setNamedColor(QStringLiteral("#efefef"));
+    paletteChanged_ = color != winColor_;
+
+    color = baseColor_;
+    baseColor_.setNamedColor(settings.value(QLatin1String("base_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = baseColor_.isValid() && color != baseColor_;
+
+    color = highlightColor_;
+    highlightColor_.setNamedColor(settings.value(QLatin1String("highlight_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = highlightColor_.isValid() && color != highlightColor_;
+
+    color = winTextColor_;
+    winTextColor_.setNamedColor(settings.value(QLatin1String("window_text_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = winTextColor_.isValid() && color != winTextColor_;
+
+    color = textColor_;
+    textColor_.setNamedColor(settings.value(QLatin1String("text_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = textColor_.isValid() && color != textColor_;
+
+    color = highlightedTextColor_;
+    highlightedTextColor_.setNamedColor(settings.value(QLatin1String("highlighted_text_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = highlightedTextColor_.isValid() && color != highlightedTextColor_;
+
+    color = linkColor_;
+    linkColor_.setNamedColor(settings.value(QLatin1String("link_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = linkColor_.isValid() && color != linkColor_;
+
+    color = linkVisitedColor_;
+    linkVisitedColor_.setNamedColor(settings.value(QLatin1String("link_visited_color")).toString());
+    if (!paletteChanged_)
+        paletteChanged_ = linkVisitedColor_.isValid() && color != linkVisitedColor_;
+
+    if(paletteChanged_)
+    {
+        delete LXQtPalette_;
+        // This sets all colors appropriately but valid custom colors are set below.
+        // If a custom color is not valid, Qt's calculated color will be used.
+        LXQtPalette_ = new QPalette(winColor_);
+
+        if (baseColor_.isValid())
+        {
+            LXQtPalette_->setColor(QPalette::Base, baseColor_);
+            // See Qt -> qpalette.cpp -> qt_fusionPalette()
+            LXQtPalette_->setColor(QPalette::Disabled, QPalette::Base, winColor_);
+        }
+        if (highlightColor_.isValid())
+        {
+            LXQtPalette_->setColor(QPalette::Highlight, highlightColor_);
+            int gray = qGray(highlightColor_.rgb());
+            color = QColor(gray, gray, gray);
+            LXQtPalette_->setColor(QPalette::Disabled, QPalette::Highlight, color);
+        }
+        else
+        {
+            // Qt's default highlight color and that of Fusion may be different. This is a workaround:
+            LXQtPalette_->setColor(QPalette::Highlight, QColor(60, 140, 230));
+            if (highlightedTextColor_.isValid())
+                LXQtPalette_->setColor(QPalette::HighlightedText, QColor(255, 255, 255));
+        }
+        if (winTextColor_.isValid())
+        {
+            LXQtPalette_->setColor(QPalette::WindowText, winTextColor_);
+            LXQtPalette_->setColor(QPalette::ButtonText, winTextColor_);
+            color = winTextColor_;
+            color.setAlpha(130);
+            LXQtPalette_->setColor(QPalette::Disabled, QPalette::WindowText, color);
+            LXQtPalette_->setColor(QPalette::Disabled, QPalette::ButtonText, color);
+        }
+        if (textColor_.isValid())
+        {
+            LXQtPalette_->setColor(QPalette::Text, textColor_);
+            color = textColor_;
+            color.setAlpha(130);
+            LXQtPalette_->setColor(QPalette::Disabled, QPalette::Text, color);
+        }
+        if (highlightedTextColor_.isValid())
+        {
+            LXQtPalette_->setColor(QPalette::HighlightedText, highlightedTextColor_);
+            color = highlightedTextColor_;
+            color.setAlpha(130);
+            LXQtPalette_->setColor(QPalette::Disabled, QPalette::HighlightedText, color);
+        }
+        if (linkColor_.isValid())
+            LXQtPalette_->setColor(QPalette::Link, linkColor_);
+        if (linkVisitedColor_.isValid())
+            LXQtPalette_->setColor(QPalette::LinkVisited, linkVisitedColor_);
+    }
+    settings.endGroup();
+
     // load Qt settings
     settings.beginGroup(QLatin1String("Qt"));
 
@@ -173,11 +276,20 @@ void LXQtPlatformTheme::onSettingsChanged() {
 
     loadSettings(); // reload the config file
 
-    if(style_ != oldStyle) // the widget style is changed
+    if(style_ != oldStyle || paletteChanged_) // the widget style or palette is changed
     {
         // ask Qt5 to apply the new style
-        if (qobject_cast<QApplication *>(QCoreApplication::instance()))
+        if(auto app = qobject_cast<QApplication *>(QCoreApplication::instance()))
+        {
             QApplication::setStyle(style_);
+            // Qt 5.15 needs this and it's safe otherwise
+            if(LXQtPalette_ != nullptr)
+            {
+                QApplication::setPalette(*LXQtPalette_);
+                // the app should be polished because the style may have an internal palette
+                QApplication::style()->polish(app);
+            }
+        }
     }
 
     if(iconTheme_ != oldIconTheme) { // the icon theme is changed
@@ -217,7 +329,7 @@ void LXQtPlatformTheme::onSettingsChanged() {
 
 bool LXQtPlatformTheme::usePlatformNativeDialog(DialogType type) const {
     if(type == FileDialog
-       && qobject_cast<QApplication *>(QCoreApplication::instance())) { // QML may not have qApp
+       && (qobject_cast<QApplication *>(QCoreApplication::instance()) != nullptr)) { // QML may not have qApp
         // use our own file dialog
         return true;
     }
@@ -227,7 +339,7 @@ bool LXQtPlatformTheme::usePlatformNativeDialog(DialogType type) const {
 
 QPlatformDialogHelper *LXQtPlatformTheme::createPlatformDialogHelper(DialogType type) const {
     if(type == FileDialog
-       && qobject_cast<QApplication *>(QCoreApplication::instance())) { // QML may not have qApp
+       && (qobject_cast<QApplication *>(QCoreApplication::instance()) != nullptr)) { // QML may not have qApp
         // use our own file dialog provided by libfm
 
         // When a process has this environment set, that means glib event loop integration is disabled.
@@ -247,7 +359,7 @@ QPlatformDialogHelper *LXQtPlatformTheme::createPlatformDialogHelper(DialogType
 
             // try to resolve the symbol to get the function pointer
             createFileDialogHelper = reinterpret_cast<CreateFileDialogHelperFunc>(libfmQtLibrary.resolve("createFileDialogHelper"));
-            if(!createFileDialogHelper) {
+            if(createFileDialogHelper == nullptr) {
                 return nullptr;
             }
         }
@@ -258,7 +370,11 @@ QPlatformDialogHelper *LXQtPlatformTheme::createPlatformDialogHelper(DialogType
     return nullptr;
 }
 
-const QPalette *LXQtPlatformTheme::palette(Palette /*type*/) const {
+const QPalette *LXQtPlatformTheme::palette(Palette type) const {
+    if(type == QPlatformTheme::SystemPalette) {
+        if(LXQtPalette_ != nullptr)
+            return LXQtPalette_;
+    }
     return nullptr;
 }
 
@@ -356,7 +472,7 @@ QIconEngine *LXQtPlatformTheme::createIconEngine(const QString &iconName) const
 {
     return new XdgIconLoaderEngine(iconName);
 }
- 
+
 // Helper to return the icon theme paths from XDG.
 QStringList LXQtPlatformTheme::xdgIconThemePaths() const
 {
@@ -379,7 +495,11 @@ QStringList LXQtPlatformTheme::xdgIconThemePaths() const
     xdgDirs.append(xdgDataDirs);
 
     for (const auto &s: xdgDirs) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
+        const QStringList r = s.split(QLatin1Char(':'), Qt::SkipEmptyParts);
+#else
         const QStringList r = s.split(QLatin1Char(':'), QString::SkipEmptyParts);
+#endif
         for (const auto& xdgDir: r) {
             const QFileInfo xdgIconsDir(xdgDir + QStringLiteral("/icons"));
             if (xdgIconsDir.isDir())
